@@ -11,6 +11,7 @@ class OperationalError (Exception):
 	
 class DBConnect:
 	zombie_timeout = 120
+	keep_alive = 120
 	
 	def __init__ (self, address, params = None, lock = None, logger = None):
 		self.address = address
@@ -36,6 +37,7 @@ class DBConnect:
 		
 		# need if there's not any request yet
 		self.active = 0
+		self.retried = 0
 		self.request = None
 		self.has_result = False
 		self.__history = []
@@ -43,6 +45,11 @@ class DBConnect:
 		
 		self.set_event_time ()
 	
+	def duplicate (self):
+		new_asyncon = self.__class__ (self.address, self.params, self.lock, self.logger)
+		new_asyncon.keep_alive = self.keep_alive
+		return new_asyncon
+		
 	def get_proto (self):
 		# call by culster_manager
 		return None
@@ -51,24 +58,23 @@ class DBConnect:
 		# call by dist_call
 		self.close ()
 		
-	def close (self):		
+	def close (self, deactive = 1):
 		self.request = None
-		self.set_active (False)		
 		addr = type (self.address) is tuple and ("%s:%d" % self.address) or str (self.address)
 		self.logger ("[info] .....dbo %s has been closed" % addr)	
+		if deactive:
+			self.set_active (False)
 					
 	def get_history (self):
 		return self.__history
 		
-	def duplicate (self):
-		return self.__class__ (self.address, self.params, self.lock, self.logger)
-		
 	def clean_shutdown_control (self, phase, time_in_this_phase):
 		self.__no_more_request = True
 		if self.isactive ():
 			return 1
 		else:
 			self.handle_close ()
+			self.__no_more_request = False
 			return 0
 	
 	def empty_cursor (self):
@@ -92,22 +98,31 @@ class DBConnect:
 		self.connect ()
 	
 	def disconnect (self):
-		self.close ()
+		# keep request, just close temporary
+		request = self.request
+		self.close (deactive = 0)
+		self.request = request
 	
 	def close_case (self):
 		raise NotImplementedError
 			
 	def set_active (self, flag, nolock = False):
+		if not flag: 
+			self.set_timeout (self.keep_alive)
+			self.retried = 0
+			
 		if flag:
 			flag = time.time ()
 		else:
 			flag = 0
+			
 		if nolock or self.lock is None:
 			self.active = flag
-			return			
+			return
+		
 		self.lock.acquire ()
 		self.active = flag
-		self.request_count += 1
+		self.request_count += 1		
 		self.lock.release ()
 	
 	def get_active (self, nolock = False):
@@ -134,7 +149,7 @@ class DBConnect:
 	def connect (self, force = 0):
 		raise NotImplementedError("must be implemented in subclass")
 	
-	def set_zombie_timeout (self, timeout):
+	def set_timeout (self, timeout):
 		self.zombie_timeout = timeout
 
 	def handle_timeout (self):
@@ -142,11 +157,19 @@ class DBConnect:
 	
 	def handle_error (self):
 		dummy, exception_class, exception_str, tbinfo = asyncore.compact_traceback()
+		self.has_result = False
 		self.logger.trace ()
 		self.handle_close (exception_class, exception_str)
 	
-	def handle_close (self, expt = None, msg = ""):		
-		self.exception_class, self.exception_str = expt, msg		
+	def handle_close (self, expt = None, msg = ""):
+		if not expt and not self.retried:			
+			self.disconnect ()
+			self.retried = 1
+			self.execute (self.request)			
+			return
+		
+		if self.exception_class is None:	
+			self.exception_class, self.exception_str = expt, msg		
 		self.close_case_with_end_tran ()
 		self.close ()
 	
@@ -170,8 +193,12 @@ class DBConnect:
 		raise NotImplementedError
 						
 	def begin_tran (self, request):
-		if self.__no_more_request:
-			raise SystemError ("Entered Shutdown Process")
+		if self.__no_more_request:			
+			try: 
+				raise OperationalError ("Entered Shutdown Process")
+			except: 
+				self.handle_error ()
+				return
 		
 		self.request = request	
 		self.__history = []
