@@ -18,7 +18,7 @@ class SynConnect (asynpsycopg2.AsynConnect, dbconnect.DBConnect):
 	def is_channel_in_map (self, map = None):
 		return False
 			
-	def close (self):	
+	def close (self, deactive = 1):	
 		if self.cur:
 			self.cur.close ()
 			self.cur = None
@@ -26,8 +26,11 @@ class SynConnect (asynpsycopg2.AsynConnect, dbconnect.DBConnect):
 			self.conn.close ()			
 			self.conn = None	
 		self.connected = False	
-		dbconnect.DBConnect.close (self)
+		dbconnect.DBConnect.close (self, deactive)
 	
+	def del_channel (self, map=None):
+		pass
+				
 	def close_case (self):
 		asynpsycopg2.AsynConnect.close_case (self)
 				
@@ -46,10 +49,10 @@ class SynConnect (asynpsycopg2.AsynConnect, dbconnect.DBConnect):
 			self.connect ()
 		
 		tranaction = False
-		sql= request.params [0].strip ()
+		sql = request.params [0].strip ()
 		try:
 			if sql [:7].lower () == "select ":
-				self.cur.execute (sql)
+				self.cur.execute (*request.params)
 			else:	
 				tranaction = True
 				self.cur.executescript (sql)
@@ -58,6 +61,5 @@ class SynConnect (asynpsycopg2.AsynConnect, dbconnect.DBConnect):
 			self.handle_error ()
 		else:
 			if tranaction: self.conn.commit ()
-			self.has_result = True		
+			self.has_result = True
 			self.close_case ()
-		
\ No newline at end of file
