@@ -1,36 +1,41 @@
 import re
-from aquests.protocols.http import http_date
-from aquests.lib import compressors
+from . import http_date
+from ...lib import compressors, attrdict
 import time
 import json
 import struct
-from aquests.protocols.grpc import discover
+from ..grpc import discover
+from ..grpc.buffers import grpc_buffer
 from . import buffers, treebuilder
 from . import localstorage as ls
+from hashlib import md5
+from urllib.parse import urljoin
+from . import urlinfo
 
 try:
 	from cStringIO import StringIO as BytesIO
 except ImportError:
 	from io import BytesIO
 
-class HTTPRepsonseError (Exception): 
+class HTTPRepsonseError (Exception):
 	pass
 
 RESPONSE = re.compile ('HTTP/([0-9.]+) ([0-9]{3})\s?(.*)')
+DEFAULT_PORT_MAP = {'http': 80, 'https': 443, 'ws': 80, 'wss': 443}
+	
 def crack_response (data):
 	global RESPONSE
 	[ version, code, msg ] = RESPONSE.findall(data)[0]
 	return version, int(code), msg
-
-
+		
 class Response:
 	SIZE_LIMIT = 2**19
 	
-	def __init__ (self, request, header):		
+	def __init__ (self, request, header):	
 		self.request = request
 		if header [:2] == "\r\n":
 			header = header [2:]
-		header = header.split ("\r\n")	
+		header = header.split ("\r\n")
 		self.response = header [0]
 		self.header = header [1:]
 		self._header_cache = {}
@@ -40,10 +45,15 @@ class Response:
 		self.max_age = 0
 		self.decompressor = None
 		self.is_xmlrpc_return = False
+		self._uinf = None
 		self.__headerdict = None
 		self.__encoding = None
 		self.__data_cache = None
+		self.__lxml = None
 		self.save_cookies ()
+	
+	def __repr__ (self):
+		return "<Response [%d]>" % self.status_code
 		
 	def set_max_age (self):
 		self.max_age = 0
@@ -102,19 +112,19 @@ class Response:
 		self.set_max_age ()
 		ct = self.get_header ("content-type", "")
 		if ct.startswith ('application/grpc'):
-			self.p, self.u = buffers.getfakeparser (buffers.grpc_buffer, cache = self.max_age)
+			self.p, self.u = buffers.getfakeparser (grpc_buffer, cache = self.max_age)
 		elif ct == 'text/xml' and self.request.xmlrpc_serialized ():
 			self.p = self.u = buffers.cachable_xmlrpc_buffer (self.max_age)
 			self.is_xmlrpc_return = True
 		else:			
 			self.p, self.u = buffers.getfakeparser (buffers.bytes_buffer, cache = self.max_age)
-					
+		
 		if self.get_header ("Content-Encoding") == "gzip":			
 			self.decompressor = compressors.GZipDecompressor ()
 			
-	def collect_incoming_data (self, data):
+	def collect_incoming_data (self, data):		
 		if self.size == 0:
-			self.init_buffer ()		
+			self.init_buffer ()
 		self.size += len (data)
 		
 		if self.decompressor:
@@ -139,7 +149,7 @@ class Response:
 		for each in v2:
 			if not each: continue
 			each = each.strip ()
-			print (each)
+			#print (each)
 			try:
 				a, b = each.split ("=", 1)
 			except ValueError:
@@ -178,7 +188,7 @@ class Response:
 			raise SystemError ("Cookie Storage Not Initiated")
 		
 	def json (self):
-		return json.loads (self.raw.read ())
+		return json.loads (self.raw.read ().decode ("utf8"))
 	
 	def raise_for_status (self):
 		if self.status_code >= 300:
@@ -196,12 +206,29 @@ class Response:
 		for line in self.header:
 			if line [:12].lower() == 'set-cookie: ':
 				ls.g.set_cookie_from_string (self.url, line [12:])
+	
+	def resolve (self, url):
+		return urljoin (self.url, url.strip ())
+	
+	def is_same (self, *args, **karg):
+		return urlinfo.uuid (*args, **karg) == self.uuid
 		
+	def __nonzero__ (self):
+		return self.status_code < 300 and self.data and True or False
+	
+	@property
+	def meta (self):
+		return self.request.meta
+			
 	@property
 	def cookies (self):
 		if ls.g:
 			return ls.g.get_cookie_as_dict (self.url)
 		raise SystemError ("Cookie Storage Not Initiated")
+	
+	@property
+	def history (self):		
+		return self.request.get_history ()
 		
 	@property
 	def url (self):		
@@ -231,7 +258,7 @@ class Response:
 	def headers (self):
 		if self.__headerdict:
 			return self.__headerdict
-		headerdict = {}
+		headerdict = attrdict.NocaseDict ()
 		for line in self.header:
 			k, v = line.split (": ", 1)
 			if k in headerdict:
@@ -241,7 +268,7 @@ class Response:
 			else:		
 				headerdict [k] = v
 		self.__headerdict = headerdict	
-		return headerdict		
+		return headerdict
 	
 	@property
 	def raw (self):
@@ -249,7 +276,10 @@ class Response:
 	
 	@property
 	def content (self):
-		return self.raw.read ()
+		try:
+			return self.raw.read ()
+		except AttributeError:
+			return b''	
 	
 	@property
 	def binary (self):
@@ -264,9 +294,12 @@ class Response:
 		self.__encoding = value
 	
 	@property
-	def dom (self):
+	def lxml (self):
+		if self.__lxml:
+			return self.__lxml			
 		if treebuilder.HAS_SKILLSET:
-			return treebuilder.html (self.raw, self.request.uri, self.encoding)
+			self.__lxml = treebuilder.html (self.content, self.request.uri, self.encoding)
+			return self.__lxml
 			
 	@property
 	def data (self):
@@ -289,14 +322,14 @@ class Response:
 			return result
 			
 		elif ct.startswith ("application/json"):
-			return json.loads (result)
+			return json.loads (result.decode ("utf8"))
 			
 		elif ct.startswith ('application/grpc'):			
 			msgs = []
 			for msg in result:
 				descriptor, isstream = discover.find_output (self.request.path [1:])					
 				f = descriptor ()
-				f.ParseFromString (msg)
+				f.ParseFromString (msg.decode ("utf8"))
 				msgs.append (f)
 								
 			if not isstream:
@@ -306,7 +339,24 @@ class Response:
 		self.__data_cache = result	
 		return result
 	
-
+	@property
+	def uuid (self, include_data = True):	
+		ct = self.request.get_header ('content-type')
+		data = ct and ct.startswith ('application/x-www-form-urlencoded') and self.request.payload.decode ('utf8') or None
+		return urlinfo.uuid (self.url, self.request.method, data, include_data)		
+	
+	@property
+	def usid (self):	
+		return urlinfo.usid (self.url, self.request.method)		
+	
+	@property
+	def uinf (self):
+		if self._uinf:
+			return self._uinf
+		self._uinf = urlinfo.uinf (self.url)
+		return self._uinf
+	
+	
 class FailedResponse (Response):
 	def __init__ (self, errcode, msg, request = None):
 		self.version, self.code, self.msg, self.header = "1.0", errcode, msg, []
@@ -314,6 +364,23 @@ class FailedResponse (Response):
 		self.buffer = None
 		self.got_all_data = True
 		self.max_age = 0
+		self._uinf = None
+		self._header_cache = {}
+	
+	@property
+	def data (self):
+		return None
+	
+	@property
+	def content (self):
+		return b''
+	
+	@property
+	def text (self):
+		return ''
+	
+	def json (self):
+		raise ValueError ('Response Failed')
 				
 	def collect_incoming_data (self, data):
 		raise IOError("This Is Failed Response")
@@ -323,11 +390,4 @@ class FailedResponse (Response):
 	
 	def done (self):
 		pass
-	
-	@property	
-	def content (self):
-		return b""
-	
-	@property	
-	def raw (self):
-		return b""	
+
